  /*
  Copyright (C) 2017 SAS Institute Inc. Cary, NC, USA
*/
/**
   \file
   \brief   Macro partition input table based on cardinality using ByN method.


   <b> Identified Inputs </b>

    \param[in] IN_DS                  Input data set that needs the partition
    \param[in] IN_CARDINALITY_DS      cardinality table
    \param[in] INDEX                  Index

   <b> Identified Outputs </b>

    \param[out] OUT_PARTITION_DS       Output instrument table after partition.


    \ingroup data_prep

    \author  SAS Institute Inc.

    \date    2017
   */
   
%macro irm_partition_byn(IN_DS=,
                         IN_CARDINALITY_DS=,
                         INDEX=&RANK,
                         OUT_PARTITION_DS=);

   %local obs_start obs_end obs_option partition_no;

   /* Get the max rank of partitions */
   data null;
      set &IN_CARDINALITY_DS.;
      call symput('partition_no', trim(left(MAX_RANK_NO)));
   run;
   
   /* if cardinality=1, no slicing needed*/
   %if &partition_no.=1 %then %do;
   
      data &OUT_PARTITION_DS.;
         set &IN_DS. ;
      run;
      
   %end;
   
   %else %do;
   
      proc sql noprint;
         select count(*) into :obs_num
            from &IN_DS.;
      quit;
      
      data work.part(drop=REMAIN_OBS ADD_FLG);
         set &IN_CARDINALITY_DS.;
         N_OBS_PER_PARTITION=int(&obs_num./MAX_RANK_NO);
         REMAIN_OBS=mod(&obs_num, MAX_RANK_NO);
         ADD_FLG=&INDEX.-REMAIN_OBS;
         if REMAIN_OBS ne 0 then do;
            if ADD_FLG <= 0 then do;
               START_OBS=(N_OBS_PER_PARTITION+1) * (&INDEX. - 1)+1;
               END_OBS=(N_OBS_PER_PARTITION+1) * &INDEX.;
            end;
            else do;
               START_OBS=N_OBS_PER_PARTITION * (&INDEX. - 1)+1+REMAIN_OBS;
               END_OBS=(N_OBS_PER_PARTITION) * &INDEX.+REMAIN_OBS;
            end;
         end;
         else do;
            START_OBS=N_OBS_PER_PARTITION * (&INDEX. - 1)+1;
            END_OBS=N_OBS_PER_PARTITION * &INDEX.;
         end;
      run;

      /* Find the first observation for this partition */
      proc sql noprint;
         select START_OBS, END_OBS into :obs_start, :obs_end
          from work.part;
      quit;

      /* Find the last observation for this partition.
         Do not specify last observation for the partition with the max rank*/

      %if &INDEX. eq &partition_no. %then %do;
         %let obs_option=;
      %end;
      %else %do;
         %let obs_option=obs %str(=) &obs_end.;
      %end;

      data &OUT_PARTITION_DS.;
         set &IN_DS. (firstobs=&obs_start. &obs_option.);
      run;
      
   %end;

%mend;
